@@ -43,7 +43,6 @@ using Banshee.Sources;
 using Banshee.ServiceStack;
 using Banshee.Streaming;
 
-        using System.Diagnostics;
 // Disabling "is never used" warnings here because there are a lot
 // of properties/fields that are set via reflection at the database
 // layer - that is, they really are used, but the compiler doesn't
@@ -61,15 +60,13 @@ namespace Banshee.Collection.Database
 
     public class DatabaseTrackInfo : TrackInfo
     {
-        private static DatabaseTrackModelProvider<DatabaseTrackInfo> provider = new DatabaseTrackModelProvider<DatabaseTrackInfo> (
-            ServiceManager.DbConnection
-        );
+        private static DatabaseTrackModelProvider<DatabaseTrackInfo> provider;
 
         public static DatabaseTrackModelProvider<DatabaseTrackInfo> Provider {
-            get { return provider; }
+            get { return provider ?? provider = new DatabaseTrackModelProvider<DatabaseTrackInfo> (ServiceManager.DbConnection); }
         }
 
-        private bool? artist_changed = null, album_changed = null;
+        private bool artist_changed = false, album_changed = false;
         private bool uri_fields_dirty = false;
         
         public DatabaseTrackInfo () : base ()
@@ -99,21 +96,40 @@ namespace Banshee.Collection.Database
 
         public override bool TrackEqual (TrackInfo track)
         {
-            if (!(track is DatabaseTrackInfo)) {
-                return base.TrackEqual (track);
+            if (PrimarySource != null && PrimarySource.TrackEqualHandler != null) {
+                return PrimarySource.TrackEqualHandler (this, track);
             }
             
-            DatabaseTrackInfo db_track = (DatabaseTrackInfo)track;
-            return db_track.TrackId == TrackId && db_track.CacheModelId == CacheModelId && 
-                db_track.CacheEntryId == CacheEntryId;
+            DatabaseTrackInfo db_track = track as DatabaseTrackInfo;
+            if (db_track == null) {
+                return base.TrackEqual (track);
+            }
+            return TrackEqual (this, db_track);
+        }
+
+        public override string ArtworkId {
+            get {
+                if (PrimarySource != null && PrimarySource.TrackArtworkIdHandler != null) {
+                    return PrimarySource.TrackArtworkIdHandler (this);
+                }
+                return base.ArtworkId;
+            }
+        }
+
+        public static bool TrackEqual (DatabaseTrackInfo a, DatabaseTrackInfo b)
+        {
+            return a != null && b != null && 
+                a.TrackId == b.TrackId && 
+                a.CacheModelId == b.CacheModelId && 
+                (int)a.CacheEntryId == (int)b.CacheEntryId;
         }
         
         public DatabaseArtistInfo Artist {
             get { return DatabaseArtistInfo.FindOrCreate (ArtistName); }
         }
-        
+
         public DatabaseAlbumInfo Album {
-            get { return DatabaseAlbumInfo.FindOrCreate (Artist, AlbumTitle); }
+            get { return DatabaseAlbumInfo.FindOrCreate (DatabaseArtistInfo.FindOrCreate (AlbumArtist), AlbumTitle, IsCompilation); }
         }
 
         private static bool notify_saved = true;
@@ -208,6 +224,11 @@ namespace Banshee.Collection.Database
         }
 
         [VirtualDatabaseColumn ("Name", "CoreArtists", "ArtistID", "ArtistID")]
+        protected string ArtistNameField {
+            get { return ArtistName; }
+            set { base.ArtistName = value; }
+        }
+
         public override string ArtistName {
             get { return base.ArtistName; }
             set {
@@ -216,11 +237,16 @@ namespace Banshee.Collection.Database
                     return;
 
                 base.ArtistName = value;
-                artist_changed = artist_changed != null;
+                artist_changed = true;
             }
         }
-        
+
         [VirtualDatabaseColumn ("Title", "CoreAlbums", "AlbumID", "AlbumID")]
+        protected string AlbumTitleField {
+            get { return AlbumTitle; }
+            set { base.AlbumTitle = value; }
+        }
+
         public override string AlbumTitle {
             get { return base.AlbumTitle; }
             set {
@@ -229,7 +255,39 @@ namespace Banshee.Collection.Database
                     return;
 
                 base.AlbumTitle = value;
-                album_changed = album_changed != null;
+                album_changed = true;
+            }
+        }
+
+        [VirtualDatabaseColumn ("ArtistName", "CoreAlbums", "AlbumID", "AlbumID")]
+        protected string AlbumArtistField {
+            get { return AlbumArtist; }
+            set { base.AlbumArtist = value; }
+        }
+
+        public override string AlbumArtist {
+            get { return base.AlbumArtist; }
+            set {
+                value = CleanseString (value, AlbumArtist);
+                if (value == null)
+                    return;
+
+                base.AlbumArtist = value;
+                album_changed = true;
+            }
+        }
+        
+        [VirtualDatabaseColumn ("IsCompilation", "CoreAlbums", "AlbumID", "AlbumID")]
+        protected bool IsCompilationField {
+            get { return IsCompilation; }
+            set { base.IsCompilation = value; }
+        }
+
+        public override bool IsCompilation {
+            get { return base.IsCompilation; }
+            set {
+                base.IsCompilation = value;
+                album_changed = true;
             }
         }
         
@@ -240,10 +298,7 @@ namespace Banshee.Collection.Database
                     
             if (input != null)
                 input = input.Trim ();
-                
-            if (input == String.Empty)
-                return null;
-                
+            
             if (input == old_val)
                 return null;
             
@@ -275,7 +330,7 @@ namespace Banshee.Collection.Database
         [DatabaseColumn ("Uri")]
         protected string UriField {
             get {
-                if (uri_fields_dirty) {
+                if (uri_fields_dirty && Uri != null && PrimarySource != null) {
                     PrimarySource.UriToFields (Uri, out uri_type, out uri_field);
                     uri_fields_dirty = false;
                 }
@@ -292,7 +347,7 @@ namespace Banshee.Collection.Database
         [DatabaseColumn ("UriType")]
         protected TrackUriType UriType {
             get {
-                if (uri_fields_dirty) {
+                if (uri_fields_dirty && Uri != null && PrimarySource != null) {
                     PrimarySource.UriToFields (Uri, out uri_type, out uri_field);
                     uri_fields_dirty = false;
                 }
@@ -316,6 +371,18 @@ namespace Banshee.Collection.Database
             get { return base.FileSize; }
             set { base.FileSize = value; }
         }
+
+        [DatabaseColumn]
+        public override long FileModifiedStamp {
+            get { return base.FileModifiedStamp; }
+            set { base.FileModifiedStamp = value; }
+        }
+
+        [DatabaseColumn]
+        public override DateTime LastSyncedStamp {
+            get { return base.LastSyncedStamp; }
+            set { base.LastSyncedStamp = value; }
+        }
         
         [DatabaseColumn ("Attributes")]
         public override TrackMediaAttributes MediaAttributes {
@@ -333,6 +400,11 @@ namespace Banshee.Collection.Database
         protected string TitleLowered {
             get { return TrackTitle == null ? null : TrackTitle.ToLower (); }
         }
+
+        [DatabaseColumn(Select = false)]
+        public override string MetadataHash {
+            get { return base.MetadataHash; }
+        }
         
         [DatabaseColumn]
         public override int TrackNumber {
@@ -346,10 +418,16 @@ namespace Banshee.Collection.Database
             set { base.TrackCount = value; }
         }
         
+        [DatabaseColumn ("Disc")]
+        public override int DiscNumber {
+            get { return base.DiscNumber; }
+            set { base.DiscNumber = value; }
+        }
+
         [DatabaseColumn]
-        public override int Disc {
-            get { return base.Disc; }
-            set { base.Disc = value; }
+        public override int DiscCount {
+            get { return base.DiscCount; }
+            set { base.DiscCount = value; }
         }
         
         [DatabaseColumn]
@@ -376,6 +454,18 @@ namespace Banshee.Collection.Database
             set { base.Composer = value; }
         }
 
+        [DatabaseColumn]
+        public override string Conductor {
+            get { return base.Conductor; }
+            set { base.Conductor = value; }
+        }
+
+        [DatabaseColumn]
+        public override string Grouping {
+            get { return base.Grouping; }
+            set { base.Grouping = value; }
+        }
+
         [DatabaseColumn]
         public override string Copyright {
             get { return base.Copyright; }
@@ -394,9 +484,26 @@ namespace Banshee.Collection.Database
             set { base.Comment = value; }
         }
         
-        [DatabaseColumn ("Rating")]
-        private int rating;
+        [DatabaseColumn("BPM")]
+        public override int Bpm {
+            get { return base.Bpm; }
+            set { base.Bpm = value; }
+        }
+
+        [DatabaseColumn]
+        public override int BitRate {
+            get { return base.BitRate; }
+            set { base.BitRate = value; }
+        }
+        
+        [DatabaseColumn("Rating")]
+        protected int rating;
         public override int Rating {
+            get { return rating; }
+            set { rating = value; }
+        }
+
+        public int SavedRating {
             get { return rating; }
             set {
                 if (rating != value) {
@@ -418,6 +525,23 @@ namespace Banshee.Collection.Database
             set { base.SkipCount = value; }
         }
         
+        private long external_id;
+        [DatabaseColumn ("ExternalID")]
+        public long ExternalId {
+            get { return external_id; }
+            set { external_id = value; }
+        }
+
+        private object external_object;
+        public override object ExternalObject {
+            get {
+                if (external_id > 0 && external_object == null && PrimarySource != null && PrimarySource.TrackExternalObjectHandler != null) {
+                    external_object = PrimarySource.TrackExternalObjectHandler (this);
+                }
+                return external_object;
+            }
+        }
+        
         [DatabaseColumn ("LastPlayedStamp")]
         public override DateTime LastPlayed {
             get { return base.LastPlayed; }
@@ -444,7 +568,7 @@ namespace Banshee.Collection.Database
         }
         
         [DatabaseColumn ("LastStreamError")]
-        private StreamPlaybackError playback_error;
+        protected StreamPlaybackError playback_error;
         public override StreamPlaybackError PlaybackError {
             get { return playback_error; }
             set {
@@ -453,7 +577,6 @@ namespace Banshee.Collection.Database
                 }
                 
                 playback_error = value; 
-                Save ();
             }
         }
 
@@ -463,7 +586,7 @@ namespace Banshee.Collection.Database
                 Uri = PrimarySource.UriAndTypeToSafeUri (UriType, UriField);
             }
         }
-        
+
         public void CopyToLibraryIfAppropriate (bool force_copy)
         {
             SafeUri old_uri = this.Uri;
@@ -472,10 +595,10 @@ namespace Banshee.Collection.Database
                 return;
             }
             
-            bool in_library = old_uri.AbsolutePath.StartsWith (Paths.CachedLibraryLocation);
+            bool in_library = old_uri.AbsolutePath.StartsWith (Paths.CachedLibraryLocationWithSeparator);
 
             if (!in_library && (LibrarySchema.CopyOnImport.Get () || force_copy)) {
-                string new_filename = FileNamePattern.BuildFull (this, Path.GetExtension (old_uri.ToString ()).Substring (1));
+                string new_filename = FileNamePattern.BuildFull (this, Path.GetExtension (old_uri.ToString ()));
                 SafeUri new_uri = new SafeUri (new_filename);
 
                 try {
@@ -503,20 +626,22 @@ namespace Banshee.Collection.Database
 
         public static int GetTrackIdForUri (string relative_path, int [] primary_sources)
         {
-            return ServiceManager.DbConnection.Query<int> (check_command,
-                primary_sources, relative_path, relative_path);
+            return GetTrackIdForUri (relative_path, relative_path, primary_sources);
+        }
+        
+        public static int GetTrackIdForUri (string uri, string relative_path, int [] primary_sources)
+        {
+            return ServiceManager.DbConnection.Query<int> (check_command, primary_sources, uri, relative_path);
         }
         
         public static bool ContainsUri (string relative_path, int [] primary_sources)
         {
-            return ServiceManager.DbConnection.Query<int> (check_command,
-                primary_sources, relative_path, relative_path) > 0;
+            return GetTrackIdForUri (relative_path, primary_sources) > 0;
         }
 
         public static bool ContainsUri (SafeUri uri, string relative_path, int [] primary_sources)
         {
-            return ServiceManager.DbConnection.Query<int> (check_command,
-                primary_sources, relative_path, uri.AbsoluteUri) > 0;
+            return GetTrackIdForUri (uri, relative_path, primary_sources) > 0;
         }
     }
 }
