@@ -39,8 +39,6 @@ namespace Banshee.Gui.TrackEditor
         private ListStore model;
         private TreeView view;
         
-        public event EventHandler Changed;
-
         public StatisticsPage ()
         {
             ShadowType = ShadowType.In;
@@ -127,6 +125,7 @@ namespace Banshee.Gui.TrackEditor
                 
                 foreach (TagLib.ICodec codec in file.Properties.Codecs) {
                     if (codec != null) {
+                        /* Translators: {0} is the description of the codec */
                         AddItem (String.Format (Catalog.GetString ("{0} Codec:"), 
                             codec.MediaTypes.ToString ()), codec.Description);
                     }
